// file: lat_comp_1.cc
//

// system include files
//
#include <string.h>
#include <memory.h>
#include <ctype.h>

// isip include files
//
#include "lattice.h"
#include "lattice_constants.h"

// method: compute_posterior_cc
//
// arguments:
//  FILE* fp_in_a, Hash_table* lexicon_a
//
// return: a logical_1 indicating status
//
// this method computes confidence estimate the links in the lattice
//
logical_1 Lattice::compute_posterior_cc(FILE* fp_in_a, Hash_table* lexicon_a) {
  
  // get the null word pointer from the lexicon
  //
  if ((lexicon_a->hash_lookup_cc(WRD_NULL)) != (Hash_cell*)NULL) {
    null_word_d =
      (Word*)((lexicon_a->hash_lookup_cc(WRD_NULL))->get_item_cc());
  }
 
  // define some local variables
  //
  char_1* buffer = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* buffer_pos = buffer;
  char_1* temp_buf = (char_1*)NULL;
  char_1* delimiter = (char_1*)"=";

  float_4 time = (float_4)0;
  int_4 num_nodes = (int_4)0;
  int_4 num_arcs = (int_4)0;

  int_4 frame_index = (int_4)0;
  int_4 node_index = (int_4)0;
  int_4 arc_count = (int_4)0;

  int_4 start_node = (int_4)0;
  int_4 end_node = (int_4)0;

  logical_1 read_words = ISIP_TRUE;
  int_4 pron_var = (int_4)0;

  float_4 ac_like = (float_4)0;
  float_4 lm_score = (float_4)0;
  float_4 lm_scale = (float_4)0;
  float_4 word_penalty = (float_4)0;

  Word* word = (Word*)NULL;
  Hash_cell* hcell = (Hash_cell*)NULL;
  Lattice_node** lnodes = (Lattice_node**)NULL;
  
  // define memory
  //
  memset((char*)buffer, 0, ISIP_MAX_STRING_LENGTH);
  
  // read in the lines till we get to reading the link/arc information
  // read the lattice generation specific information like utterance
  // number, word insertion penalty, lexicon, and model set
  // also read the node information
  //
  while (fgets((char*)buffer_pos, ISIP_MAX_STRING_LENGTH, fp_in_a) &&
	 (*buffer_pos != 'J')) {
    
    // find the delimited value
    //
    temp_buf = (char_1*)strtok((char*)buffer_pos, (char*)delimiter);
    
    // parse this string
    //
    while (temp_buf != (char_1*)NULL) {
      
      // move the pointer to first non-whitespace
      //
      while (isspace((char)(*temp_buf))) {
	temp_buf++;
      }
      
      // read the lattice generation information
      // version number of generation program (typically HVite)
      //
      if (strcmp((char*)temp_buf, "VERSION") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // utterance represented by the lattice
      //
      else if (strcmp((char*)temp_buf, "UTTERANCE") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	set_utterance_cc((char_1*)temp_buf);
      }
      
      // word insertion penalty used during lattice generation
      //
      else if (strcmp((char*)temp_buf, "wdpenalty") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	word_penalty = atof((char*)temp_buf);
	set_word_penalty_cc(word_penalty);
      }
      
      // language model scaling used during lattice generation
      //
      else if (strcmp((char*)temp_buf, "lmscale") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	lm_scale = atof((char*)temp_buf);
	set_lm_scale_cc(lm_scale);
      }
      
      // language model used during lattice generation
      //
      else if (strcmp((char*)temp_buf, "vocab") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	set_lang_model_cc((char_1*)temp_buf);
      }
      
      // model set used during lattice generation
      //
      else if (strcmp((char*)temp_buf, "hmms") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	set_model_set_cc((char_1*)temp_buf);
      }
      
      // number of nodes in the lattice
      //
      else if (strcmp((char*)temp_buf, "N") == 0) {
	
	// do not confuse with alphabet N which could be a word
	//
	if (read_words == ISIP_TRUE) {

	  // set the number of lattice nodes
	  //
	  temp_buf = (char_1*)strtok((char*)NULL, " ");
	  num_nodes = atoi((char*)temp_buf);
	  set_num_nodes_cc(num_nodes);
	  
	  // initialize the lattice nodes table
	  //
	  lnodes = new Lattice_node*[num_nodes];
	  init_nodes_cc(lnodes);
  	}
	
	// otherwise ignore this word and move on
	//
	else {
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	}
      }
      
      // number of arcs in the lattice
      //
      else if (strcmp((char*)temp_buf, "L") == 0) {
	
	// do not confuse with alphabet L which could be a word
	//
	if (read_words == ISIP_TRUE) {

	  // set the number of arcs
	  //
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	  num_arcs = atoi((char*)temp_buf);
	  set_num_arcs_cc(num_arcs);
	}

	// otherwise ignore this word and move on
	//
	else {
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	}
      }
      
      // read the node information (node index, end time, word etc.)
      //
      else if (strcmp((char*)temp_buf, "I") == 0) {
	
	// do not confuse with alphabet I which could be a word
	//
	if (read_words == ISIP_TRUE) {

	  // set the node index
	  //
	  temp_buf = (char_1*)strtok((char*)NULL, " ");
	  node_index = atoi((char*)temp_buf);
	  lnodes[node_index]->set_node_index_cc(node_index);
	}

	// otherwise ignore this word and move on
	//
	else {
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	}
      }

      // read the frame information for this node
      //
      else if (strcmp((char*)temp_buf, "t") == 0) {
	
	// do not confuse with alphabet t which could be a word
	//
	if (read_words == ISIP_TRUE) {

	  // set the frame index
	  //
	  temp_buf = (char_1*)strtok((char*)NULL, " ");
	  time = atof((char*)temp_buf);
	  frame_index = (int_4)rint(time * LATTICE_FRAMES_PER_SEC);
	  lnodes[node_index]->set_frame_index_cc(frame_index);
	}

	// otherwise ignore this word and move on
	//
	else {
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	}
      }
      
      // read the word index of the word represented by the node
      //
      else if (strcmp((char*)temp_buf, "W") == 0) {
	
	// get the word string and check the lexicon if this word
	// exists
	//
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	hcell = lexicon_a->hash_lookup_cc((char_1*)temp_buf);
	
	// error if word not found in lexicon
	//
	if (hcell == (Hash_cell*)NULL) {
	  fprintf (stdout, "%s\n", temp_buf);
	  error_handler_cc((char_1*)"lat_read_0.cc",
			   (char_1*)"Word does not exist in lexicon");
	}
	
	// set the item index of the node
	//
	word = (Word*)(hcell->get_item_cc());
	lnodes[node_index]->set_word_cc(word);
	read_words = ISIP_FALSE;
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // read in the pronunciation variant used
      //
      else if (strcmp((char*)temp_buf, "v") == 0) {
	
	// do not confuse with alphabet L which could be a word
	//
	if (read_words == ISIP_TRUE) {
	  
	  // set the pronunciation variation index
	  //
	  temp_buf = (char_1*)strtok((char*)NULL, " ");
	  pron_var = atoi((char*)temp_buf);
	  lnodes[node_index]->set_pron_var_cc(pron_var);
	}
	
	// otherwise ignore this word and move on
	//
	else {
	  temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	}
      }
      
      // otherwise ignore this word and move on
      //
      else {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
	read_words = ISIP_TRUE;
      }
    } // end while temp_buf is not null
  } // end while fgets
  
  // start reading the arc information
  // complete parsing the first line of the arc information
  // loop over the remaining lines of the file
  //
  do {
    
    // find the delimited value
    //
    temp_buf = (char_1*)strtok((char*)buffer_pos, (char*)delimiter);
    
    // while the line is not empty
    //
    while (temp_buf != (char_1*)NULL) {
      
      // move the pointer to first non-whitespace
      //
      while (isspace((char)(*temp_buf))) {
	temp_buf++;
      }
      
      // read in the arc index
      //
      if (strcmp((char*)temp_buf, "J") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	arc_count++;
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // start node of arc
      //
      else if (strcmp((char*)temp_buf, "S") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	start_node = atoi((char*)temp_buf);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // end node of arc
      //
      else if (strcmp((char*)temp_buf, "E") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	end_node = atoi((char*)temp_buf);
	
	// update the prev node and next node pointers for the two nodes
	// corresponding to this arc
	//
	lnodes[start_node]->add_next_node_cc(lnodes[end_node]);
	lnodes[start_node]->add_next_node_index_cc(end_node);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // word represented by the arc end node
      //
      else if (strcmp((char*)temp_buf, "W") == 0) {
	
	// find the corresponding word in the lexicon
	//
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	hcell = lexicon_a->hash_lookup_cc((char_1*)temp_buf);
	
	// error if word not found in lexicon
	//
	if (hcell == (Hash_cell*)NULL) {
	  fprintf (stdout, "%s\n", (char*)temp_buf);
	  error_handler_cc((char_1*)"lat_read_0.cc",
			   (char_1*)"Word does not exist in lexicon");
	}
	
	// set the item index of the node
	//
	word = (Word*)(hcell->get_item_cc());
	lnodes[end_node]->set_word_cc(word);
	
	// create the lattice start node --- this is a dummy node that is
	// not evaluated, it just serves as a starting point for all the
	// sentence start nodes
	//
	if (strcmp((char*)temp_buf, (char*)LATTICE_SENT_START) == 0) {
	  start_node_d = lnodes[start_node];
	}
	else if (strcmp((char*)temp_buf, (char*)LATTICE_NET_START) == 0) {
	  start_node_d = lnodes[start_node];
	}
	
	// move on to the next word
	//
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // pronunciation variant
      //
      else if (strcmp((char*)temp_buf, "v") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	pron_var = atoi((char*)temp_buf);
	lnodes[end_node]->set_pron_var_cc(pron_var);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // acoustic likelihood of arc
      //
      else if (strcmp((char*)temp_buf, "a") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	ac_like = atof((char*)temp_buf);
	lnodes[start_node]->add_ac_score_cc(ac_like);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // read in the lm score
      //
      else if (strcmp((char*)temp_buf, "l") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	lm_score = atof((char*)temp_buf);
	lnodes[start_node]->add_lm_score_cc(lm_score);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // move on to the next word
      //
      else {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
    }
  } while ((fgets((char*)buffer_pos,ISIP_MAX_STRING_LENGTH,fp_in_a)) &&
	   (arc_count <= num_arcs_d));
  
  // get the previous node indexes from the lattice node
  // datastructure, to do this we should have first built the lattice
  // node structure for all the nodes
  //
  rewind(fp_in_a);
  
  // redefine some local variables
  //
  buffer_pos = buffer;
  temp_buf = (char_1*)NULL;
  
  start_node = (int_4)0;
  end_node = (int_4)0;
  arc_count = (int_4)0;
  
  // define memory
  //
  memset((char*)buffer, 0, ISIP_MAX_STRING_LENGTH);
  
  // start reading the arc information
  // complete parsing the first line of the arc information
  //
  do {

    // find the delimited value
    //
    temp_buf = (char_1*)strtok((char*)buffer_pos, (char*)delimiter);
    
    // while the line is not empty
    //
    while (temp_buf != (char_1*)NULL) {
      
      // move the pointer to first non-whitespace
      //
      while (isspace((char)(*temp_buf))) {
	temp_buf++;
      }

      // read in the arc index
      //
      if (strcmp((char*)temp_buf, "J") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	arc_count++;
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // start node of arc
      //
      else if (strcmp((char*)temp_buf, "S") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	start_node = atoi((char*)temp_buf);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // end node of arc
      //
      else if (strcmp((char*)temp_buf, "E") == 0) {
	temp_buf = (char_1*)strtok((char*)NULL, " ");
	end_node = atoi((char*)temp_buf);
	
	// update the prev node and next node pointers for the two nodes
	// corresponding to this arc
	//
	lnodes[end_node]->add_prev_node_cc(lnodes[start_node]);
	lnodes[end_node]->add_prev_node_index_cc(start_node);
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
      
      // move on to the next word
      //
      else {
	temp_buf = (char_1*)strtok((char*)NULL, (char*)delimiter);
      }
    }
  } while ((fgets((char*)buffer_pos,ISIP_MAX_STRING_LENGTH,fp_in_a)) &&
	   (arc_count <= num_arcs_d));

  // compute the posteriors on every link
  //

  // declare local variables
  //
  float_4 alpha_temp = (float_4)0;
  float_4 beta_temp = (float_4)0;
  float_4 sum_temp = (float_4)0;
  float_4 next_node_value_temp = (float_4)0;
  float_4 final_set_value_temp = (float_4)0;
  float_4 prev_node_value_temp = (float_4)0;
  Word* curr_word;
  Word* temp_word;
  //char_1* curr_wdname = (char_1*)NULL;
  //char_1* temp_wdname = (char_1*)NULL;
  float_4 final_posterior = (float_4)0;
  //float_4 accum_similar_posterior = (float_4)0;
  //float_4 accum_dissimilar_posterior = (float_4)0;
  int_4 start_time_frame;
  int_4 end_time_frame;
  int_4 time_duration;
  int_4 mid_time;
  
  // initialize the alpha, beta and posterior values
  //
  for (int_4 i = 0; i < num_nodes_d; i++) {
    lnodes[i]->set_alpha_cc((float_4)0);
    lnodes[i]->set_beta_cc((float_4)0);
  }
  
  // compute forward probabilities
  //
  for (int_4 i = 0; i < num_nodes_d - 1; i++) {
    int_4 num_arcs = lnodes[i]->get_num_arcs_out_cc();
    float_4* ac_temp = new float_4[num_arcs];
    float_4* lm_temp = new float_4[num_arcs];
    int_4* next_node_index_temp = new int_4[num_arcs];
    memcpy(ac_temp,lnodes[i]->get_ac_scores_cc(),(num_arcs) * sizeof(float_4));
    memcpy(lm_temp,lnodes[i]->get_lm_scores_cc(),(num_arcs) * sizeof(float_4));
    memcpy(next_node_index_temp,lnodes[i]->get_next_node_index_cc(),(num_arcs) * sizeof(float_4));
    for (int_4 j = 0; j < num_arcs; j++) {
      alpha_temp = lnodes[i]->get_alpha_cc();
      sum_temp = alpha_temp + ((1/lm_scale_d)*ac_temp[j]) + lm_temp[j];
      next_node_value_temp = lnodes[next_node_index_temp[j]]->get_alpha_cc();
      if (next_node_value_temp == (float_4)0) {
	final_set_value_temp = sum_temp;
      }
      else {

	float tmp;
	
	// find the minimum component so that the sign of tmp is negative
	// below
	//
	if (next_node_value_temp <  sum_temp) {
	  
	  tmp = next_node_value_temp;
	  sum_temp = next_node_value_temp;
	  next_node_value_temp = tmp;
	}
	
	// find the difference between the two values. the output of
	// this will always be negative
	//
	tmp = next_node_value_temp - sum_temp;
	
	if (tmp > -23.02585093) {
	  
	  // this "trick comes from the comments in the source code
	  // for FDLIBM, SunSoft's Freely Distributable Math Library,
	  // www.netlib.org/fdlibm.  They reference the HP-15C
	  // Advanced Functions Handbook.  The idea probably comes
	  // from Prof. William Kahan, U. C. Berkeley.
	  //
	  float x;
	  x = 1 + tmp;
	  if (x = (float)1) {
	    final_set_value_temp = tmp;
	  }
	  else {
	    final_set_value_temp = log(x) * (tmp / x - 1);
	  }
	}
	//final_set_value_temp = sum_temp + log(1+exp(next_node_value_temp - sum_temp));
      }
      lnodes[next_node_index_temp[j]]->set_alpha_cc(final_set_value_temp);
    }
    delete [] next_node_index_temp;
    delete [] ac_temp;
    delete [] lm_temp;
  }
  
  // compute backward probabilities
  //
  int_4 exact_num_nodes_d = num_nodes_d - 1;
  for (int_4 i = 0; i < num_nodes_d - 1; i++) {
    int_4 num_arcs = lnodes[exact_num_nodes_d - i]->get_num_arcs_in_cc();
    int_4* prev_node_index_temp = new int_4[num_arcs];
    memcpy(prev_node_index_temp,lnodes[exact_num_nodes_d - i]->get_prev_node_index_cc(),(num_arcs) * sizeof(float_4));
    float_4* ac_temp;
    float_4* lm_temp;
    int_4* next_node_index_temp;
    
    for (int_4 j = 0; j < num_arcs; j++) {
      beta_temp = lnodes[exact_num_nodes_d - i]->get_beta_cc();
      int_4 num_arcs_out = lnodes[prev_node_index_temp[j]]->get_num_arcs_out_cc();
      ac_temp = new float_4[num_arcs_out];
      lm_temp = new float_4[num_arcs_out];
      memcpy(ac_temp,lnodes[prev_node_index_temp[j]]->get_ac_scores_cc(),(num_arcs_out) * sizeof(float_4));
      memcpy(lm_temp,lnodes[prev_node_index_temp[j]]->get_lm_scores_cc(),(num_arcs_out) * sizeof(float_4));
      next_node_index_temp = new int_4[num_arcs_out];
      memcpy(next_node_index_temp,lnodes[prev_node_index_temp[j]]->get_next_node_index_cc(),(num_arcs_out) * sizeof(float_4));
      int_4 corresponding_backtrace;
      for (int_4 k = 0; k < num_arcs_out; k++) {
	if (next_node_index_temp[k] = (exact_num_nodes_d - i)) {
	  corresponding_backtrace = k;
	  break;
	}
      }
      prev_node_value_temp = lnodes[prev_node_index_temp[j]]->get_beta_cc();
      sum_temp = beta_temp +
	((1/lm_scale_d)*ac_temp[corresponding_backtrace]) +
	lm_temp[corresponding_backtrace];
      if (prev_node_value_temp == (float_4)0) {
	final_set_value_temp = sum_temp;
      }
      else {

	float tmp;
	
	// find the minimum component so that the sign of tmp is negative
	// below
	//
	if (prev_node_value_temp <  sum_temp) {
	  
	  tmp = prev_node_value_temp;
	  sum_temp = prev_node_value_temp;
	  prev_node_value_temp = tmp;
	}
	
	// find the difference between the two values. the output of
	// this will always be negative
	//
	tmp = prev_node_value_temp - sum_temp;
	
	if (tmp > -23.02585093) {
	  
	  // this "trick comes from the comments in the source code
	  // for FDLIBM, SunSoft's Freely Distributable Math Library,
	  // www.netlib.org/fdlibm.  They reference the HP-15C
	  // Advanced Functions Handbook.  The idea probably comes
	  // from Prof. William Kahan, U. C. Berkeley.
	  //
	  float x;
	  x = 1 + tmp;
	  if (x = (float)1) {
	    final_set_value_temp = tmp;
	  }
	  else {
	    final_set_value_temp = log(x) * (tmp / x - 1);
	  }
	}
	
	//final_set_value_temp = sum_temp + log(1+exp(prev_node_value_temp - sum_temp));
      }
      lnodes[prev_node_index_temp[j]]->set_beta_cc(final_set_value_temp);
      delete [] ac_temp;
      delete [] lm_temp;
      delete [] next_node_index_temp;
    }
    delete [] prev_node_index_temp;
  }
  
  // compute arc posteriors
  //
  for (int_4 i = 0; i < num_nodes_d - 1;i++) {
    int_4 num_arcs = lnodes[i]->get_num_arcs_out_cc();
    float_4* ac_temp = new float_4[num_arcs];
    float_4* lm_temp = new float_4[num_arcs];
    int_4* next_node_index_temp = new int_4[num_arcs];
    memcpy(ac_temp,lnodes[i]->get_ac_scores_cc(),(num_arcs) * sizeof(float_4));
    memcpy(lm_temp,lnodes[i]->get_lm_scores_cc(),(num_arcs) * sizeof(float_4));
    memcpy(next_node_index_temp,lnodes[i]->get_next_node_index_cc(),(num_arcs) * sizeof(float_4));
    for (int_4 j = 0; j < num_arcs; j++) {
      alpha_temp = lnodes[i]->get_alpha_cc();
      sum_temp = alpha_temp - ac_temp[j] + lm_temp[j];
      next_node_value_temp = lnodes[next_node_index_temp[j]]->get_beta_cc();

      // the normalization is performed by dividing the numerator by the
      // sum of all paths which in this case will be the alpha value at the
      // last node
      //
      final_set_value_temp = sum_temp + next_node_value_temp - lnodes[num_nodes - 1]->get_alpha_cc();
      
      // time normalize by dividing by the word duration
      //
      start_time_frame = lnodes[i]->get_frame_index_cc();
      end_time_frame = lnodes[next_node_index_temp[j]]->get_frame_index_cc();
      time_duration = abs(end_time_frame - start_time_frame);
      final_set_value_temp = (final_set_value_temp / time_duration);
      lnodes[i]->add_gamma_cc(final_set_value_temp,j);
    }
    delete [] ac_temp;
    delete [] lm_temp;
    delete [] next_node_index_temp;
  }
  
  // in order to estimate the confidence using the posteriors, we will have to
  // sum accross all the links with similar words at a particular time instant.
  // the time instant chosen for each link is the mid point of the link.
  //
  for (int_4 i = 0; i < num_nodes_d - 1; i++) {
    
    int_4 num_arcs = lnodes[i]->get_num_arcs_out_cc();
    int_4* next_node_index_temp = new int_4[num_arcs];
    float_4* conf_estimate = new float_4[num_arcs];
    memcpy(next_node_index_temp,lnodes[i]->get_next_node_index_cc(),(num_arcs) * sizeof(float_4));
    memcpy(conf_estimate,lnodes[i]->get_gamma_cc(),(num_arcs) * sizeof(float_4));
    for (int_4 j = 0; j < num_arcs; j++) {
      
      start_time_frame = lnodes[i]->get_frame_index_cc();
      end_time_frame = lnodes[next_node_index_temp[j]]->get_frame_index_cc();
      time_duration = abs(end_time_frame - start_time_frame);
      mid_time = time_duration / 2;
      curr_word = lnodes[next_node_index_temp[j]]->get_word_cc();
      float_4 total_confidence = (float_4)0;
      int_4 count = (int_4)0;
      
      for (int_4 k = 0; k < num_nodes_d - 1; k++) {
	
	int_4 num_arcs_search = lnodes[k]->get_num_arcs_out_cc();
	int_4* next_node_index_temp_search = new int_4[num_arcs_search];
	memcpy(next_node_index_temp_search, lnodes[k]->get_next_node_index_cc(),(num_arcs_search) * sizeof(float_4));
	start_time_frame = lnodes[k]->get_frame_index_cc();
	
	for (int_4 l = 0; l < num_arcs_search; l++) {
	  
	  start_time_frame = lnodes[k]->get_frame_index_cc();
	  
	  // since the nodes are sorted in time we can break out of the loop
	  // once the start node goes beyond the mid-time
	  //
	  if (start_time_frame > mid_time) {
	    break;
	  }
	  end_time_frame = lnodes[next_node_index_temp_search[l]]->get_frame_index_cc();
	  temp_word = lnodes[next_node_index_temp_search[l]]->get_word_cc();
	  if ((temp_word == curr_word) && (end_time_frame > mid_time)) {
	    if (conf_estimate[l] > total_confidence) {
	      total_confidence = conf_estimate[l];
	    }
	    count++;
	  }
	}
	
	// clear memory
	//
	delete [] next_node_index_temp_search;
      }
      if (total_confidence == (float_4)0) {
	lnodes[i]->add_posterior_cc((-conf_estimate[j]), j);
      }
      else {
	//total_confidence = total_confidence / count;
	lnodes[i]->add_posterior_cc((-total_confidence), j);
      }
    }
    
    // clear memory
    //
    delete [] next_node_index_temp;
    delete [] conf_estimate;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
