// file: lat_get_10.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "lattice.h"
#include "lattice_constants.h"

// method: get_arc_posterior_cc
//
// arguments:
//  * lnode: (input) the lattice node
//  Hash_cell*& lcell: (output) the hash cell containing this lattice node
//  
// return: a logical_1 indicating success
//
// this method gets the hash cell containing the given lattice node
//
float_4 Lattice::get_arc_posterior_cc(char_1* word_name_a,
				      int_4 end_node_index_a,
				      int_4 start_time_frame_a,
				      int_4 end_time_frame_a,
				      float_4 score_a) {

  // local variables
  //
  float_4 posterior;
  Lattice_node* start_latn = start_node_d;
  Lattice_node* end_latn = (Lattice_node*)NULL;
  
  // initialize a list of lattice nodes
  //
  Lattice_node** lnodes = (Lattice_node**)NULL;
  lat_arrange_cc(lnodes);

  // get the number of incominf arcs for the end node
  //
  int_4 num_arcs_in = lnodes[end_node_index_a]->get_num_arcs_in_cc();
  
  // get previous node indexes
  //
  int_4* prev_node_indexes = new int_4[num_arcs_in];
  memcpy(prev_node_indexes,lnodes[end_node_index_a]->get_prev_node_index_cc(),(num_arcs_in) * sizeof(float_4));
  
  // loop over all previous nodes to find which node corresponds to
  // the same start time
  //
  for (int_4 i = 0; i < num_arcs_in; i++) {
    int_4 start_time = lnodes[prev_node_indexes[i]]->get_frame_index_cc();
    if (start_time == start_time_frame_a) {
      //((int_4)(lnodes[prev_node_indexes[i]]->get_ac_scores_cc()[i]) == (int_4)(score_a - word_penalty_d))) {
      int_4 num_arcs_out = lnodes[prev_node_indexes[i]]->get_num_arcs_out_cc();
      int_4* next_node_indexes = new int_4[num_arcs_out];
      memcpy(next_node_indexes,lnodes[prev_node_indexes[i]]->get_next_node_index_cc(),(num_arcs_out) * sizeof(float_4));
      
      // obtain the posterior value corresponding to the particular arc
      //
      for (int_4 j = 0; j < num_arcs_out; j++) {
	if (lnodes[next_node_indexes[j]]->get_node_index_cc() == end_node_index_a) {
	  posterior = (float_4)lnodes[prev_node_indexes[i]]->get_confidence_cc()[j];		      
	  break;
	}
      }
      
      // clear memory
      //
      delete [] next_node_indexes;
    }
  }
  
  // clear memory
  //
  delete [] prev_node_indexes;
  
  // exit gracefully
  //
  return posterior;
}
