// file: latn_comp_0.cc
//

// system include files
//
#include <string.h>
#include <memory.h>
#include <ctype.h>

// isip include files
//
#include "lattice_node.h"
#include "lattice_node_constants.h"

// method: compute_prob_cc
//
// arguments:
//  Lattice_node** lnodes_a
//  int_4 num_nodes_a
//  float_4 lm_scale_a
//
// return: a logical_1 indicating status
//
// this method computes the forward-backward probabilities for the lattice
//
logical_1 Lattice_node::compute_prob_cc(Lattice_node** lnodes_a,
					int_4 num_nodes_a,
					float_4 lm_scale_a) {
  
  // declare local variables
  //
  float_4 alpha_temp;
  float_4 beta_temp;
  float_4 sum_temp;
  float_4 next_node_value_temp;
  float_4 final_set_value_temp;
  float_4 prev_node_value_temp;
  
  // initialize the alpha, beta and posterior values
  //
  for (int_4 i = 0; i < num_nodes_a; i++) {
    lnodes_a[i]->set_alpha_cc((float_4)-23.02585093);
    lnodes_a[i]->set_beta_cc((float_4)-23.02585093);
  }
  
  // compute forward probabilities
  //
  for (int_4 i = 0; i < num_nodes_a; i++) {
    float_4* ac_temp;
    int_4* next_node_index_temp;
    int_4 num_arcs = lnodes_a[i]->get_num_arcs_out_cc();
    for (int_4 j = 0; j < num_arcs; j++) {
      alpha_temp = lnodes_a[i]->get_alpha_cc();
      ac_temp = lnodes_a[i]->get_ac_scores_cc();
      sum_temp = alpha_temp + ((1/lm_scale_a)*ac_temp[j]);
      next_node_index_temp = lnodes_a[i]->get_next_node_index_cc();
      next_node_value_temp = lnodes_a[next_node_index_temp[j]]->get_alpha_cc();
      if (next_node_value_temp = (float_4)-23.02585093) {
	final_set_value_temp = sum_temp;
      }
      else {
	final_set_value_temp = sum_temp + log(1+exp(next_node_value_temp - sum_temp));
      }
      lnodes_a[next_node_index_temp[j]]->set_alpha_cc(final_set_value_temp);
    }
  }
  
  // compute backward probabilities
  //
  int_4 exact_num_nodes = num_nodes_a - 1;
  for (int_4 i = 0; i < num_nodes_a; i++) {
    int_4* prev_node_index_temp;
    float_4* ac_temp;
    int_4* next_node_index_temp;
    int_4 num_arcs = lnodes_a[exact_num_nodes - i]->get_num_arcs_in_cc();
    for (int_4 j = 0; j < num_arcs; j++) {
      beta_temp = lnodes_a[exact_num_nodes - i]->get_beta_cc();
      prev_node_index_temp = lnodes_a[exact_num_nodes - i]->get_prev_node_index_cc();
      ac_temp = lnodes_a[prev_node_index_temp[j]]->get_ac_scores_cc();
      next_node_index_temp = lnodes_a[prev_node_index_temp[j]]->get_next_node_index_cc();
      int_4 num_arcs_out = lnodes_a[prev_node_index_temp[j]]->get_num_arcs_out_cc();
      int_4 corresponding_backtrace;
      for (int_4 k = 0; k < num_arcs_out; k++) {
	if (next_node_index_temp[k] = (exact_num_nodes - i)) {
	  corresponding_backtrace = k;
	  break;
	}
      }
      prev_node_value_temp = lnodes_a[prev_node_index_temp[j]]->get_beta_cc();
      sum_temp = beta_temp + ((1/lm_scale_a)*ac_temp[corresponding_backtrace]);
      if (prev_node_value_temp = (float_4)-23.02585093) {
	final_set_value_temp = sum_temp;
      }
      else {
	final_set_value_temp = sum_temp + log(1+exp(prev_node_value_temp - sum_temp));
      }
      lnodes_a[prev_node_index_temp[j]]->set_beta_cc(final_set_value_temp);
    }
  }
  
  // compute arc posteriors
  //
  for (int_4 i = 0; i < num_nodes_a - 1;i++) {
    float_4* ac_temp;
    int_4* next_node_index_temp;
    int_4 num_arcs = lnodes_a[i]->get_num_arcs_out_cc();
    int_4 start_time_frame;
    int_4 end_time_frame;
    int_4 time_duration;
    for (int_4 j = 0; j < num_arcs; j++) {
      alpha_temp = lnodes_a[i]->get_alpha_cc();
      ac_temp = lnodes_a[i]->get_ac_scores_cc();
      sum_temp = alpha_temp + ac_temp[j];
      next_node_index_temp = lnodes_a[i]->get_next_node_index_cc();
      next_node_value_temp = lnodes_a[next_node_index_temp[j]]->get_beta_cc();
      final_set_value_temp = sum_temp + next_node_value_temp;
      
      // normalize by dividing by the word duration
      //
      start_time_frame = lnodes_a[i]->get_frame_index_cc();
      end_time_frame = lnodes_a[next_node_index_temp[j]]->get_frame_index_cc();
      time_duration = abs(end_time_frame - start_time_frame);
      final_set_value_temp = (final_set_value_temp / time_duration);
      lnodes_a[i]->add_gamma_cc(final_set_value_temp,j);
    }
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
