// file: tr_lxt_tree_3.cc
//

// isip include files
//
#include "train_lex_tree.h"
#include "train_lex_tree_constants.h"
#include <string.h>
// method: build_tree_cc
//
// arguments:
//  Train_Lattice_node* lat_node : (input) the lattice node for the word to add
//  float_8 score : (input) the word transition score for this word
//
// return: a logical_1 to indicate success
//
// this method loops over all pronunciations of the input word, for
// each pronunciation it checks phone-by-phone if the current phone is
// already on the correct branch of the tree and inserts the word and
// score in the corresponding node
//
logical_1 Train_Lex_tree::build_train_tree_cc(char_1* phn_list_a,
				  char_1** monophones_a,  int_4 num_mono_a,
				  Train_Lattice_node* lat_node_a) {
  
  // phone / word parameters
  //
  int_4 phone = (int_4)-1;
  int_4 num_ph = (int_4)0;
  head_d = new Train_Lex_node; 
  float_8 default_score = (float_8)0;
  
  int_4* phones = new int_4[ISIP_MAX_STRING_LENGTH];
 
  char_1* tmp = new char_1[ISIP_MAX_STRING_LENGTH];

  //
  // read data from file
  //
  num_ph = (int_4)0;
  //while (sscanf((char*)phn_list_a, "%s", tmp) != EOF) {
  for(tmp = (char_1*)strtok((char*)phn_list_a, " "); tmp != (char_1*)NULL;
      tmp  = (char_1*)strtok(NULL, " ")) {
 
    for (int_4 k = 0; k < num_mono_a; k++) {
      if (strcmp((char*)tmp, (char*)monophones_a[k]) == 0) {
	phones[num_ph++] = k;
	break;
      }
    }
  }

  // dummy lexical nodes
  //
  Train_Lex_node* curr_node = (Train_Lex_node*)NULL;
  Train_Lex_node* next_node = (Train_Lex_node*)NULL;
  Train_Lex_node* stop_node = (Train_Lex_node*)NULL;
  Train_Lex_node* st_node = (Train_Lex_node*)NULL;

  // memory manager
  //
  Train_Memory_manager* manager = Train_Link_list::get_manager_cc();

  // set the tree pointer in the head node
  //
  if (head_d->get_tree_cc() == (void_p)NULL) {
    head_d->set_tree_cc((void_p)this);
  }

  
    // set the next node as current
    //
    stop_node = (Train_Lex_node*)NULL;
    next_node = head_d;
    
    // loop over all phones in this pronunciations
    //
    for (int_4 j = 0; j < num_ph; j++) {
      
      // set the current node
      //
      curr_node = next_node;
      
      // the jth phone of this ith pronunciation
      //
      phone = phones[j];
      
      // get the node in the tree that contains this phone
      //
      next_node = curr_node->make_node_cc(phone);
      if (next_node->get_tree_cc() == (void_p)NULL) {
	next_node->set_tree_cc(this);
      }
      
      // add the current word to the list at this node and adjust LM
      // score
      //
      next_node->add_word_cc(lat_node_a, default_score);
    }

    // set one last phone node with "sil" for the SENT_END word
    //
    stop_node = manager->new_lex_cc();
    stop_node->set_phone_cc(TRAIN_LXT_SILENCE);
    stop_node->set_tree_cc((void_p)this);
    stop_node->add_word_cc(lat_node_a, default_score);
    // insert it in the appropriate place
    //
    next_node->add_child_cc(stop_node);
    next_node = stop_node;
    
    // add the stop node at the end of word
    //
    st_node = next_node->get_node_cc(TRAIN_LXN_STOP_PHONE);
    if (st_node != (Train_Lex_node*)NULL) {
      st_node->add_word_cc(lat_node_a, default_score);
      st_node = (Train_Lex_node*)NULL;
    }
    
    else {

      // create a stop node
      //
      stop_node = manager->new_lex_cc();
      stop_node->set_phone_cc(TRAIN_LXN_STOP_PHONE);
      stop_node->set_tree_cc((void_p)this);
      stop_node->add_word_cc(lat_node_a, default_score);
      
      // insert it in the appropriate place
      //
      next_node->add_child_cc(stop_node);
    }

    // clean up memory
    //
    delete tmp;
    
    // exit gracefully
    //
    return ISIP_TRUE;  
}


